@@ -1,5 +1,5 @@
 /* Clzip - LZMA lossless data compressor
-   Copyright (C) 2010-2022 Antonio Diaz Diaz.
+   Copyright (C) 2010-2024 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -43,13 +43,14 @@ static void list_line( const unsigned long long uncomp_size,
 
 
 int list_files( const char * const filenames[], const int num_filenames,
-                const bool ignore_trailing, const bool loose_trailing )
+                const struct Cl_options * const cl_opts )
   {
   unsigned long long total_comp = 0, total_uncomp = 0;
   int files = 0, retval = 0;
   int i;
   bool first_post = true;
   bool stdin_used = false;
+
   for( i = 0; i < num_filenames; ++i )
     {
     const bool from_stdin = ( strcmp( filenames[i], "-" ) == 0 );
@@ -61,7 +62,7 @@ int list_files( const char * const filenames[], const int num_filenames,
     if( infd < 0 ) { set_retval( &retval, 1 ); continue; }
 
     struct Lzip_index lzip_index;
-    Li_init( &lzip_index, infd, ignore_trailing, loose_trailing );
+    Li_init( &lzip_index, infd, cl_opts );
     close( infd );
     if( lzip_index.retval != 0 )
       {
