@@ -1,5 +1,5 @@
 /* Clzip - LZMA lossless data compressor
-   Copyright (C) 2010-2021 Antonio Diaz Diaz.
+   Copyright (C) 2010-2024 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,13 +16,13 @@
 */
 
 #ifndef INT64_MAX
-#define INT64_MAX  0x7FFFFFFFFFFFFFFFLL
+#define INT64_MAX 0x7FFFFFFFFFFFFFFFLL
 #endif
 
 
 struct Block
   {
-  long long pos, size;			/* pos + size <= INT64_MAX */
+  long long pos, size;		/* pos >= 0, size >= 0, pos + size <= INT64_MAX */
   };
 
 static inline void init_block( struct Block * const b,
@@ -40,10 +40,10 @@ struct Member
   };
 
 static inline void init_member( struct Member * const m,
-                                const long long dp, const long long ds,
-                                const long long mp, const long long ms,
+                                const long long dpos, const long long dsize,
+                                const long long mpos, const long long msize,
                                 const unsigned dict_size )
-  { init_block( &m->dblock, dp, ds ); init_block( &m->mblock, mp, ms );
+  { init_block( &m->dblock, dpos, dsize ); init_block( &m->mblock, mpos, msize );
     m->dictionary_size = dict_size; }
 
 struct Lzip_index
@@ -58,7 +58,7 @@ struct Lzip_index
   };
 
 bool Li_init( struct Lzip_index * const li, const int infd,
-              const bool ignore_trailing, const bool loose_trailing );
+              const struct Cl_options * const cl_opts );
 
 void Li_free( struct Lzip_index * const li );
 
